(*----------------------------------------------------------------------------*
 *                                                                            *
 *         M A R S                                                            *
 *                                                                            *
 *         by Jan Strter                                                     *
 *                                                                            *
 *----------------------------------------------------------------------------*)

 unit global;

interface

uses
  Windows, Direct3D9, D3DX9, dxerr9, landscape, DXTypes;

type
  TPlane = object
    m_normal: TD3DXVector3;
    m_distance: Single;
    procedure normalize;
    function DistanceToPoint(pt: TD3DXVector3): Single;
  end;

var
    m_pd3dDevice: IDirect3DDevice9;
    terrain: CLandscape;

    // Texturen
    detail_texture: IDirect3DTexture9;
    overall_textures: array[0..15] of IDirect3DTexture9;
    heightmap, lightmap: IDirect3DTexture9;
    skytextures: array [0..5] of IDirect3DTexture9;

    // Lichtquelle (Sonne)
    sun: record
         direction: TD3DXVector3;
         color: record
            r, g, b: single;
         end;
         ambient: record
            r, g, b: single;
         end;
    end;

    // View-Port
    viewport: record
        position: TD3DXVector3;
        rotation: TD3DXVector3;
    end;
    x_versatz, y_versatz, z_versatz:single;
    m_frustumPlanes: array [0..5] of TPlane;
    planes: array[0..5] of TD3DXPlane;
    matProj: TD3DXMatrix;

    polygonCount: integer;

    fps: Integer;
    speedfactor: single;


procedure extractPlanes(matView: TD3DXMatrix);
function isVisible(aabbMin, aabbMax: TD3DXVector3): integer;
procedure initFrameCounter(fps: single);
procedure calculateSpeedFactor;

implementation

var
  targetfps: single;
  tickspersecond: int64;
  currentticks: int64;
  framedelay: int64;


procedure initFrameCounter(fps: single);
begin
     targetfps := fps;
     QueryPerformanceCounter(framedelay);
     QueryPerformanceFrequency(tickspersecond);
end;

procedure calculateSpeedFactor;
begin
     if (tickspersecond = 0) or (targetfps = 0) then exit;
     QueryPerformanceCounter(currentTicks);
     speedfactor := (currentTicks-framedelay) / (tickspersecond / targetfps);
     if speedfactor <= 0 then speedfactor := 1;
     fps := round(targetfps / speedfactor);
     framedelay := currentticks;
end;

procedure TPlane.normalize;
var denom: Single;
begin
  denom := 1 / sqrt(sqr(m_normal.x) + sqr(m_normal.y) + sqr(m_normal.z));
  m_normal.x := m_normal.x * denom;
  m_normal.y := m_normal.y * denom;
  m_normal.z := m_normal.z * denom;
  m_distance := m_distance * denom;
end;


function TPlane.DistanceToPoint(pt: TD3DXVector3): Single;
begin
  result := D3DXVec3Dot(m_normal, pt) + m_distance;
end;

procedure extractplanes;
var
   combomatrix: TD3DXMatrix;
begin
  D3DXMatrixMultiply(comboMatrix, matView, matProj);

  m_frustumPlanes[0].m_normal.x := -(comboMatrix._14 + comboMatrix._11);
  m_frustumPlanes[0].m_normal.y := -(comboMatrix._24 + comboMatrix._21);
  m_frustumPlanes[0].m_normal.z := -(comboMatrix._34 + comboMatrix._31);
  m_frustumPlanes[0].m_distance := -(comboMatrix._44 + comboMatrix._41);
  m_frustumPlanes[0].normalize;

  // Right clipping plane
  m_frustumPlanes[1].m_normal.x := -(comboMatrix._14 - comboMatrix._11);
  m_frustumPlanes[1].m_normal.y := -(comboMatrix._24 - comboMatrix._21);
  m_frustumPlanes[1].m_normal.z := -(comboMatrix._34 - comboMatrix._31);
  m_frustumPlanes[1].m_distance := -(comboMatrix._44 - comboMatrix._41);
  m_frustumPlanes[1].normalize;

  // Top clipping plane
  m_frustumPlanes[2].m_normal.x := -(comboMatrix._14 - comboMatrix._12);
  m_frustumPlanes[2].m_normal.y := -(comboMatrix._24 - comboMatrix._22);
  m_frustumPlanes[2].m_normal.z := -(comboMatrix._34 - comboMatrix._32);
  m_frustumPlanes[2].m_distance := -(comboMatrix._44 - comboMatrix._42);
  m_frustumPlanes[2].normalize;

  // Bottom clipping plane
  m_frustumPlanes[3].m_normal.x := -(comboMatrix._14 + comboMatrix._12);
  m_frustumPlanes[3].m_normal.y := -(comboMatrix._24 + comboMatrix._22);
  m_frustumPlanes[3].m_normal.z := -(comboMatrix._34 + comboMatrix._32);
  m_frustumPlanes[3].m_distance := -(comboMatrix._44 + comboMatrix._42);
  m_frustumPlanes[3].normalize;

  // Near clipping plane
  m_frustumPlanes[4].m_normal.x := -(comboMatrix._14 + comboMatrix._13);
  m_frustumPlanes[4].m_normal.y := -(comboMatrix._24 + comboMatrix._23);
  m_frustumPlanes[4].m_normal.z := -(comboMatrix._34 + comboMatrix._33);
  m_frustumPlanes[4].m_distance := -(comboMatrix._44 + comboMatrix._43);
  m_frustumPlanes[4].normalize;

  // Far clipping plane
  m_frustumPlanes[5].m_normal.x := -(comboMatrix._14 - comboMatrix._13);
  m_frustumPlanes[5].m_normal.y := -(comboMatrix._24 - comboMatrix._23);
  m_frustumPlanes[5].m_normal.z := -(comboMatrix._34 - comboMatrix._33);
  m_frustumPlanes[5].m_distance := -(comboMatrix._44 - comboMatrix._43);
  m_frustumPlanes[5].normalize;

end;

function isVisible;
var minExtreme, maxExtreme: TD3DXVector3;
    intersect: boolean;
    i: integer;
begin
  intersect := false;

  for i := 0 to 5 do
  begin
    if m_frustumPlanes[i].m_normal.x >= 0 then
    begin
      minExtreme.x := aabbMin.x;
      maxExtreme.x := aabbMax.x;
    end
    else
    begin
      minExtreme.x := aabbMax.x;
      maxExtreme.x := aabbMin.x;
    end;

    if m_frustumPlanes[i].m_normal.y >= 0 then
    begin
      minExtreme.y := aabbMin.y;
      maxExtreme.y := aabbMax.y;
    end
    else
    begin
      minExtreme.y := aabbMax.y;
      maxExtreme.y := aabbMin.y;
    end;

    if m_frustumPlanes[i].m_normal.z >= 0 then
    begin
      minExtreme.z := aabbMin.z;
      maxExtreme.z := aabbMax.z;
    end
    else
    begin
      minExtreme.z := aabbMax.z;
      maxExtreme.z := aabbMin.z;
    end;

    if m_frustumPlanes[i].DistanceToPoint(minExtreme) > 0 then
    begin
      result := 0;
      exit;
    end;
    if m_frustumPlanes[i].DistanceToPoint(maxExtreme) >= 0 then intersect := true;
  end;

  if intersect then result := 1 else result := 2;
end;

initialization
  speedFactor := 1;
end.

