(*----------------------------------------------------------------------------*
 *                                                                            *
 *         M A R S                                                            *
 *                                                                            *
 *         by Jan Strter                                                     *
 *                                                                            *
 *----------------------------------------------------------------------------*)

unit main;

interface

uses
  Windows, Messages, Direct3D9, D3DX9, dxerr9,
  D3DApp, D3DFont, D3DUtil, DXUtil, sysutils, math, directinput,
  global, skybox, landscape, player, models;

const
   MAX_STONES = 150;

type

  CLandscapeApp = class(CD3DApplication)

    // Maus- und Tastatur-Input
    m_bKey: array[#0..#255] of Boolean;
    FDirectInputObj: IDirectInput8;
    FDirectInputDevice: IDirectInputDevice8;

    // Spieler
    player: CPlayer;

    // Info
    m_pFont: CD3DFont;
    test: CModel;
    testInst :CModelInstance;

    stones_array: array[0..MAX_STONES] of CModelInstance;


  protected
    function OneTimeSceneInit: HRESULT; override;
    function InitDeviceObjects: HRESULT; override;
    function RestoreDeviceObjects: HRESULT; override;
    function InvalidateDeviceObjects: HRESULT; override;
    function DeleteDeviceObjects: HRESULT; override;
    function FinalCleanup: HRESULT; override;
    function ConfirmDevice(const pCaps: TD3DCaps9; dwBehavior: DWORD; adapterFormat: TD3DFormat; backBufferFormat: TD3DFormat): HRESULT; override;
    function FrameMove: HRESULT; override;
    function Render: HRESULT; override;
    function loadTextures: HRESULT;
    procedure destroyTextures;
    procedure setUpDirectInput;
    procedure destroyDirectInput;
    procedure processDirectInputMouse;
    procedure initLightSource;
  public
    constructor Create;
    function MsgProc(hWnd: HWND; uMsg: UINT; wParam: WPARAM; lParam: LPARAM): LRESULT; override;
  end;

///////////////////////////////////////////////////////////////////////////
implementation
///////////////////////////////////////////////////////////////////////////

const
    UnitName = 'main.pas';

var
       mouseState: TDIMouseState;

//-----------------------------------------------------------------------------
// Name: CLandscapeApp()
// Desc: Application constructor. Sets attributes for the app.
//-----------------------------------------------------------------------------
constructor CLandscapeApp.Create;
begin
  inherited;
  m_strWindowTitle := 'Landscape';
  m_bShowCursorWhenFullscreen := false;
  m_d3dEnumeration.AppUsesDepthBuffer := True;
  m_bStartFullscreen := true;

  player := CPlayer.create(10, 2.5, 0.02);
  initFrameCounter(100);
  m_pFont := CD3DFont.Create('Arial', 12, D3DFONT_BOLD);

end;


//-----------------------------------------------------------------------------
// Name: OneTimeSceneInit()
// Desc: Paired with FinalCleanup().
//       The window has been created and the IDirect3D9 interface has been
//       created, but the device has not been created yet.  Here you can
//       perform application-related initialization and cleanup that does
//       not depend on a device.
//-----------------------------------------------------------------------------
function CLandscapeApp.OneTimeSceneInit: HRESULT;
begin
  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: InitDeviceObjects()
// Desc: Paired with DeleteDeviceObjects()
//       The device has been created.  Resources that are not lost on
//       Reset() can be created here -- resources in D3DPOOL_MANAGED,
//       D3DPOOL_SCRATCH, or D3DPOOL_SYSTEMMEM.  Image surfaces created via
//       CreateImageSurface are never lost and can be created here.  Vertex
//       shaders and pixel shaders can also be created here as they are not
//       lost on Reset().
//-----------------------------------------------------------------------------
function CLandscapeApp.InitDeviceObjects:HRESULT;
var
    i, counter:integer;
    x, y, z: single;
    abbruch: boolean;
    prozentuale_hoehe: single;
    zufallszahl: single;
begin
  // Initialize the font's internal textures
  m_pFont.InitDeviceObjects(m_pd3dDevice);
  initLightSource;
  global.m_pd3dDevice := m_pd3dDevice;
  loadTextures;
  createSkybox;
  terrain := CLandscape.Create(m_pd3dDevice);
  terrain.InitGeometry;
  test := CModel.create;
  test.load('models/stein.x');
  testInst := CModelInstance.Create(test);
  testInst.setRotation(pi/2, pi/2, 0);
  testInst.setPosition(0, -250, 0);

  randomize;
  for i := 0 to MAX_STONES - 1 do begin
      stones_array[i] := CModelInstance.Create(test);

      abbruch := false;
      counter := 0;
      repeat
          x := random(3840) + x_versatz;
          z := random(3840) + z_versatz;
          y := terrain.getHeightAtPos(-x, -z);

          prozentuale_hoehe := (y + y_versatz) / (255 * heightFactor);
          zufallszahl := random;

          if (zufallszahl > prozentuale_hoehe) then begin
              abbruch := true;
          end;

          inc(counter);
          if (counter > 100) then abbruch := true;

      until abbruch;

      stones_array[i].setRotation(pi/2, pi/2, 0);
      stones_array[i].setPosition(x, -y, z);

  end;

  setUpDirectInput;
  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: RestoreDeviceObjects()
// Desc: Paired with InvalidateDeviceObjects()
//       The device exists, but may have just been Reset().  Resources in
//       D3DPOOL_DEFAULT and any other device state that persists during
//       rendering should be set here.  Render states, matrices, textures,
//       etc., that don't change during rendering can be set once here to
//       avoid redundant state setting during Render() or FrameMove().
//-----------------------------------------------------------------------------
function CLandscapeApp.RestoreDeviceObjects: HRESULT;
var
  fAspect: Single;
  m_matProjection: TD3DXMatrix;
  Light: TD3DLight9;
  Mtrl: TD3DMaterial9;
  fStart, fEnd, fDensity: single;
begin
//  global.m_pd3dDevice := m_pd3dDevice;
  loadTextures;
  m_pFont.RestoreDeviceObjects;
  fAspect := m_d3dsdBackBuffer.Width / m_d3dsdBackBuffer.Height;
  D3DXMatrixPerspectiveFovLH(m_matProjection, D3DX_PI / 2, fAspect, 1.0, 100000.0);
  matProj := m_matProjection;
  m_pd3dDevice.SetTransform(D3DTS_PROJECTION, m_matProjection);
  m_pd3dDevice.SetRenderState(D3DRS_LIGHTING, dWord(false));

  // Nebel
  fStart := 200;
  fEnd   := 1000;
  fDensity := 0.0025;
  m_pd3dDevice.SetRenderState(D3DRS_FOGENABLE, CARDINAL(TRUE));
  m_pd3dDevice.SetRenderState(D3DRS_FOGCOLOR, D3DCOLOR_RGBA(143, 52, 0, 100));
  m_pd3dDevice.SetRenderState(D3DRS_FOGSTART, Cardinal((@fStart)^));
  m_pd3dDevice.SetRenderState(D3DRS_FOGEND,   Cardinal((@fEnd)^));
  m_pd3dDevice.SetRenderState(D3DRS_FOGTABLEMODE, Cardinal(D3DFOG_LINEAR));

//  m_pd3dDevice.SetRenderState(D3DRS_FOGDENSITY, cardinal((@fDensity)^));
//  m_pd3dDevice.SetRenderState(D3DRS_FOGTABLEMODE, Cardinal(D3DFOG_EXP));
  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: InvalidateDeviceObjects()
// Desc: Invalidates device objects.  Paired with RestoreDeviceObjects()
//-----------------------------------------------------------------------------
function CLandscapeApp.InvalidateDeviceObjects: HRESULT;
begin
  m_pFont.InvalidateDeviceObjects;

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: DeleteDeviceObjects()
// Desc: Paired with InitDeviceObjects()
//       Called when the app is exiting, or the device is being changed,
//       this function deletes any device dependent objects.
//-----------------------------------------------------------------------------
function CLandscapeApp.DeleteDeviceObjects: HRESULT;
begin
  global.m_pd3dDevice := nil;
  destroyTextures;
  terrain.Destroy;
  test.destroy;
  destroyDirectInput;

  m_pFont.DeleteDeviceObjects;
  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: FinalCleanup()
// Desc: Called before the app exits, this function gives the app the chance
//       to cleanup after itself.
//-----------------------------------------------------------------------------
function CLandscapeApp.FinalCleanup: HRESULT;
begin
  player.Destroy;
  SAFE_DELETE(m_pFont);

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: ConfirmDevice()
// Desc: Called during device initialization, this code checks the device
//       for some minimum set of capabilities
//-----------------------------------------------------------------------------
function CLandscapeApp.ConfirmDevice(const pCaps: TD3DCaps9; dwBehavior: DWORD;
  adapterFormat: TD3DFormat; backBufferFormat: TD3DFormat): HRESULT;
begin
  Result:= E_FAIL;

  // Need to support post-pixel processing (for alpha blending)
  if FAILED(m_pD3D.CheckDeviceFormat(pCaps.AdapterOrdinal, pCaps.DeviceType,
       adapterFormat, D3DUSAGE_RENDERTARGET or D3DUSAGE_QUERY_POSTPIXELSHADER_BLENDING,
       D3DRTYPE_SURFACE, backBufferFormat))
  then Exit;

  // Only D3DFMT_X1R5G5B5, D3DFMT_R5G6B5, or D3DFMT_X8R8G8B8 format
  // are supported for fullscreen GDI mode.
  if not (backBufferFormat in [D3DFMT_X1R5G5B5, D3DFMT_R5G6B5, D3DFMT_X8R8G8B8]) then Exit;

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: FrameMove()
// Desc: Called once per frame, the call is the entry point for animating
//       the scene.
//-----------------------------------------------------------------------------
function CLandscapeApp.FrameMove: HRESULT;
var
   Keyboardstate: TKeyboardstate;
   matrix_x, matrix_y, matrix_z: TD3DMatrix;
   matrix: TD3DMatrix;
   trans: TD3DMatrix;
begin
    processDirectInputMouse;
    GetKeyboardState(Keyboardstate);

    If (Keyboardstate[VK_UP] And 128) > 0 Then Begin
       player.accelerateForward;
    End;
    If (Keyboardstate[VK_DOWN] And 128) > 0 Then Begin
       player.accelerateBackward;
    End;
    If (Keyboardstate[VK_LEFT] And 128) > 0 Then Begin
       player.accelerateLeft;
    End;
    If (Keyboardstate[VK_RIGHT] And 128) > 0 Then Begin
       player.accelerateRIGHT;
    End;


    if m_bKey['Z'] then
       m_pd3dDevice.SetRenderState( D3DRS_FILLMODE, D3DFILL_WIREFRAME );

    if m_bKey['U'] then
       m_pd3dDevice.SetRenderState( D3DRS_FILLMODE, D3DFILL_SOLID );




    if m_bKey['H'] then
        testInst.move(0, 4, 0);

    if m_bKey['B'] then
        testInst.move(0, -4, 0);

    if m_bKey['G'] then
        testInst.move(4, 0, 0);

    if m_bKey['J'] then
        testInst.move(-4, 0, 0);

    if m_bKey['I'] then
        testInst.rotate(pi/50, 0, 0);

    if m_bKey['O'] then
        testInst.rotate(-pi/50, 0, 0);

    if m_bKey['K'] then
        testInst.rotate(0, pi/50, 0);

    if m_bKey['L'] then
        testInst.rotate(0, -pi/50, 0);




    player.calculateNextFrame;
    viewport.position := player.position;
    viewport.rotation := player.rotation;

    // Welt-Matrix
    D3DXMatrixIdentity(matrix);
    m_pd3dDevice.SetTransform(D3DTS_WORLD, matrix);

    // Ansicht-Matrix
    D3DXMatrixRotationX(matrix_x, viewport.rotation.x);
    D3DXMatrixRotationY(matrix_y, viewport.rotation.y);
    D3DXMatrixRotationZ(matrix_z, viewport.rotation.z);

    D3DXMatrixTranslation(trans, viewport.position.x, viewport.position.y, viewport.position.z);
    D3DXMatrixMultiply(matrix, matrix, trans);

    D3DXMatrixMultiply(matrix, matrix, matrix_y);
    D3DXMatrixMultiply(matrix, matrix, matrix_z);
    D3DXMatrixMultiply(matrix, matrix, matrix_x);

    extractPlanes(matrix);

    m_pd3dDevice.SetTransform(D3DTS_VIEW, matrix);
    Result:= S_OK;
end;



//-----------------------------------------------------------------------------
// Name: Render()
// Desc: Called once per frame, the call is the entry point for 3d
//       rendering. This function sets up render states, clears the
//       viewport, and renders the scene.
//-----------------------------------------------------------------------------
function CLandscapeApp.Render: HRESULT;
var
   bla:PAnsiChar;
   matPos: TD3DXMatrix;
   i: integer;
   mapx, mapy: integer;
begin
  // Clear the viewport
  m_pd3dDevice.Clear(0, nil, D3DCLEAR_TARGET or D3DCLEAR_ZBUFFER,
                     D3DCOLOR_RGBA(202,98,41,0), 1.0, 0);

  // Begin the scene
  polygonCount := 0;
  if SUCCEEDED(m_pd3dDevice.BeginScene) then
  begin
        m_pd3dDevice.SetRenderState(D3DRS_LIGHTING, iFalse);
        renderSkybox(m_pd3dDevice);
        terrain.Render;
        m_pd3dDevice.SetRenderState(D3DRS_LIGHTING, iTrue);
//        testInst.render;

{        for i := 0 to MAX_STONES - 1 do begin
            stones_array[i].render;
        end; }

       // Output statistics
       m_pFont.DrawText(2,  0, D3DCOLOR_ARGB(255,255,255,0), m_strFrameStats);
       m_pFont.DrawText(2, 20, D3DCOLOR_ARGB(255,255,255,0), m_strDeviceStats);
       bla := PAnsiChar(inttostr(polygonCount)+' Polygone');
       m_pFont.DrawText(2, 40, D3DCOLOR_ARGB(255,255,255,0), bla);
       mapX := trunc((-player.position.x - x_Versatz) / cellWidth);
       mapY := trunc((-player.position.z - z_versatz) / cellWidth);
       bla := PAnsiChar('MapX: '+inttostr(mapx)+'  MapY: '+inttostr(mapy));
       m_pFont.DrawText(2, 60, D3DCOLOR_ARGB(255,255,255,0), bla);

       bla := PAnsiChar(inttostr(sizeof(TCustomVertex))+' FPS');
       m_pFont.DrawText(2, 80, D3DCOLOR_ARGB(255,255,255,0), bla);

       // End the scene
       m_pd3dDevice.EndScene;
  end;
  calculateSpeedFactor;

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: MsgProc()
// Desc: Message proc function to handle user input
//-----------------------------------------------------------------------------
function CLandscapeApp.MsgProc(hWnd: HWND; uMsg: UINT; wParam: WPARAM; lParam: LPARAM): LRESULT;
var
  x: Integer;
begin
  // Record key presses
  if (WM_KEYDOWN = uMsg) then
  begin
    m_bKey[Char(wParam)] := True;
  end;
  // Perform commands when keys are released
  if (WM_KEYUP = uMsg) then
  begin
    m_bKey[Char(wParam)]:= False;
  end;

  // Pass mouse messages to the ArcBall so it can build internal matrices
  Result:= inherited MsgProc(hWnd, uMsg, wParam, lParam);
end;

function CLandscapeApp.loadTextures: HRESULT;
var
   i: integer;
begin
  Result:= E_FAIL;

  // Use D3DX to create a texture from a file based image
  if (detail_texture = NIL) then begin
      if FAILED(D3DXCreateTextureFromFile(m_pd3dDevice, 'textures/detail_texture.jpg', detail_texture)) then
      begin
          exit;
      end;
  end;

  if (heightmap = NIL) then begin
      if FAILED(D3DXCreateTextureFromFile(m_pd3dDevice, 'textures/heightmap.bmp', heightmap)) then
      begin
          exit;
      end;
  end;

  if (overall_textures[0] = NIL) then begin
     for i := 0 to 15 do begin
      if FAILED(D3DXCreateTextureFromFile(m_pd3dDevice, pchar('textures/texture_' + inttostr(i + 1) + '.jpg'), overall_textures[i])) then
      begin
          exit;
      end;
     end;
  end;

  for i := 0 to 5 do begin
      if (skytextures[i] = NIL) then begin
         if FAILED(D3DXCreateTextureFromFile(m_pd3dDevice, pchar('textures/skybox' + inttostr(i + 1) + '.bmp'), skytextures[i])) then
         begin
              exit;
         end;
      end;
  end;

  Result:= S_OK;
end;

procedure CLandscapeApp.destroyTextures;
var
   i: integer;
begin
  if (detail_texture <> nil) then
    detail_texture:= nil;

  if (heightmap <> nil) then
    heightmap:= nil;

  if (lightmap <> nil) then
    lightmap:= nil;

  for i := 0 to 5 do begin
      if skytextures[i] <> NIL then
         skytextures[i] := NIL;
  end;

  for i := 0 to 15 do begin
      if (overall_textures[i] <> NIL) then
         overall_textures[i] := NIL;
  end;
end;



procedure CLandscapeApp.setUpDirectInput;
var aformat:TDIDataFormat;
begin
     aFormat := c_dfDIMouse;
     if not FAILED(DirectInput8Create(HInstance,DIRECTINPUT_VERSION, IID_IDirectInput8, FDirectInputObj, nil)) then begin
        if not FAILED(FDirectInputObj.CreateDevice(GUID_SysMouse, FDirectInputDevice, NIL)) then begin
           if FAILED(FDirectInputDevice.SetDataFormat(aFormat)) then halt;
           if FAILED(FDirectInputDevice.SetCooperativeLevel(m_hWnd, DISCL_FOREGROUND or DISCL_EXCLUSIVE)) then halt;
           if failed(FDirectInputDevice.Acquire) then halt;
        end else halt;
     end else halt;
end;

procedure CLandscapeApp.destroyDirectInput;
begin
        FDirectInputDevice.Unacquire;
     if FDirectInputDevice <> NIL then
        FDirectInputDevice := NIL;

     if FDirectInputObj <> NIL then
        FDirectInputObj := NIL;
end;

procedure CLandscapeApp.processDirectInputMouse;
const
     mouseSpeed = 0.01;
begin
   if FDirectInputDevice <> NIL then begin
        if (not failed(FDirectInputDevice.GetDeviceState(SizeOf(mouseState), @mouseState))) then begin
            with mouseState do begin
                player.setRotation(player.userRotationX - ly * mouseSpeed, player.rotation.y - lx * mouseSpeed);
            end;
        end;

     end;
end;

procedure CLandscapeApp.initLightSource;
var
  Light: TD3DLight9;
  Mtrl: TD3DMaterial9;
  lightsource: TD3DXVector3;
begin
  // Lichtquelle setzen
  lightsource.x := 1;
  lightsource.y := -1;
  lightsource.z := 1;
  D3DXVec3Normalize(lightsource, lightsource);
  sun.direction := lightsource;
  sun.color.r := 1;
  sun.color.g := 1;
  sun.color.b := 1;
  sun.ambient.r := 0.2;
  sun.ambient.g := 0.2;
  sun.ambient.b := 0.2;

  m_pd3dDevice.SetRenderState( D3DRS_SPECULARENABLE, cardinal(TRUE) );
  D3DUtil_InitLight(Light, D3DLIGHT_DIRECTIONAL, -lightsource.x, lightsource.y, -lightsource.z);
  Light.Diffuse:= D3DXColor(1, 216 / 255, 198 / 255, 1);
  Light.Specular:= D3DXColor(1, 216 / 255, 198 / 255, 1);
  Light.Ambient:= D3DXColor(0.1, 0.1, 0.1, 0.1);
  m_pd3dDevice.SetLight(0, Light);
  m_pd3dDevice.LightEnable(0, True);

{  D3DUtil_InitMaterial(Mtrl, 1, 1, 1, 1);
  m_pd3dDevice.SetMaterial(Mtrl);}

  m_pd3dDevice.SetRenderState(D3DRS_AMBIENT, $444444);

  m_pd3dDevice.SetRenderState(D3DRS_LIGHTING, iTrue);
  m_pd3dDevice.SetRenderState(D3DRS_ZENABLE, iTrue);
    
end;

end.
