unit models;

interface

uses
  Windows, Messages, Direct3D9, D3DX9, dxerr9,
  D3DUtil, DXUtil, sysutils, math, global;

type
  TD3DXMaterialsArray = array [0..0] of TD3DXMaterial;

    CModel = class
//        m_pMeshSysMem: ID3DXMesh;                // System memory copy of mesh

        m_pMesh: ID3DXMesh;                      // Local version of mesh, copied on resize
        m_pVertexBuffer: IDirect3DVertexBuffer9; // vertex buffer of mesh

        m_dwNumMaterials:   DWord; // Number of materials
        m_pMeshTextures: array of IDirect3DTexture9;
        m_pMeshMaterials: array of TD3DMaterial9;
        m_vObjectCenter:    TD3DXVector3;           // Center of bounding sphere of object
        m_fObjectRadius:    Single;                 // Radius of bounding sphere of object

        constructor Create;
        destructor destroy; override;
        function Load(m_strMeshFilename: string): HResult;
        procedure render;
    end;

    CModelInstance = class
        model: CModel;
        position: TD3DXVector3;
        rotation: TD3DXVector3;
        localMatrix: TD3DMatrix;
        isVisible: boolean;
        matrixChanged: boolean;
        constructor Create(const inst: CModel);
        destructor destroy; override;
        procedure setPosition(x, y, z: single);
        procedure move(x, y, z: single);
        procedure setRotation(x, y, z: single);
        procedure rotate(x, y, z: single);
        procedure calculateMatrix;
        procedure render;
    end;

implementation

constructor CModel.Create;
begin
//  m_pMeshSysMem:= nil;
  m_pMesh:= nil;
  m_pVertexBuffer:= nil;
end;

destructor CModel.destroy;
var
    i: integer;
begin
//    m_pMeshSysMem := nil;                // System memory copy of mesh

    m_pMesh := nil;                      // Local version of mesh, copied on resize
    m_pVertexBuffer := nil; // vertex buffer of mesh

  for i:= 0 to m_dwNumMaterials - 1 do
    m_pMeshTextures[i]:= nil;

  m_pMeshMaterials:= nil;
  m_pMeshTextures:= nil;

end;

function CModel.Load(m_strMeshFilename: string): HResult;
var
  pMeshVB:              IDirect3DVertexBuffer9;
  pD3DXMtrlBuffer:      ID3DXBuffer;
  pVertices:            Pointer;
  strMesh:              array[0..511] of Char;
  bNormalsInFile:       BOOL;
  i:                    Integer;
  d3dxMaterials:        ^TD3DXMaterialsArray;
  strPath:              array[0..511] of Char;
  tszFilename:          PChar;
  filename: string;
{$IFDEF UNICODE}
  tszBuf: array[0..MAX_PATH-1] of Char;
{$ENDIF}
begin
  pMeshVB:= nil;
  pD3DXMtrlBuffer:= nil;
  // Result:= S_OK;

  // Get a path to the media file
{  Result:= DXUtil_FindMediaFileCb(strMesh, SizeOf(strMesh), PAnsiChar(m_strMeshFilename));
  if FAILED(Result) then Exit;}

  // Load the mesh from the specified file
  Result:= D3DXLoadMeshFromX(PAnsiChar(m_strMeshFilename), D3DXMESH_SYSTEMMEM, m_pd3dDevice,
                             nil, @pD3DXMtrlBuffer, nil,
                             @m_dwNumMaterials, m_pMesh);
  if FAILED(Result) then Exit;

  // Get the array of materials out of the returned buffer, and allocate a texture array
  d3dxMaterials := pD3DXMtrlBuffer.GetBufferPointer;
  try
    SetLength(m_pMeshMaterials, m_dwNumMaterials);
    SetLength(m_pMeshTextures,  m_dwNumMaterials);
  except
    Result:= E_OUTOFMEMORY;
    Exit;
  end;

  for i:= 0 to m_dwNumMaterials - 1 do
  begin
    m_pMeshMaterials[i] := d3dxMaterials[i].MatD3D;
    m_pMeshMaterials[i].diffuse := m_pMeshMaterials[i].Diffuse;
    m_pMeshMaterials[i].specular := m_pMeshMaterials[i].Specular;
    m_pMeshMaterials[i].ambient := m_pMeshMaterials[i].Diffuse;
    m_pMeshTextures[i]  := nil;

    // Get a path to the texture
    if (d3dxMaterials[i].pTextureFilename <> nil) then
    begin
{$IFDEF UNICODE}
      tszFilename := tszBuf;
      MultiByteToWideChar(CP_ACP, 0, d3dxMaterials[i].pTextureFilename, -1, @tszBuf, MAX_PATH);
      tszBuf[MAX_PATH - 1] := #0;
{$ELSE}
      tszFilename := d3dxMaterials[i].pTextureFilename;
      filename := string(tszFilename);
      filename := 'models/textures/'+filename;
{$ENDIF}
//      DXUtil_FindMediaFileCb(strPath, SizeOf(strPath), tszFilename);

      // Load the texture
      D3DXCreateTextureFromFile(m_pd3dDevice, PAnsiChar(filename), m_pMeshTextures[i]);
    end;
  end;

  // Done with the material buffer
  SAFE_RELEASE(pD3DXMtrlBuffer);

  // Lock the vertex buffer, to generate a simple bounding sphere
{  Result:= m_pMesh.GetVertexBuffer(pMeshVB);
  if SUCCEEDED(Result) then
  begin
    Result:= pMeshVB.Lock(0, 0, pVertices, D3DLOCK_NOSYSLOCK);
    if SUCCEEDED(Result) then
    begin
      Result:= D3DXComputeBoundingSphere(PD3DXVector3(pVertices), m_pMesh.GetNumVertices,
                                         D3DXGetFVFVertexSize(m_pMesh.GetFVF),
                                         m_vObjectCenter, m_fObjectRadius);
      pMeshVB.Unlock;
    end;
    pMeshVB:= nil;
  end;
  if FAILED(Result) then Exit; }

  // remember if there were normals in the file, before possible clone operation
{  bNormalsInFile := (m_pMesh.GetFVF and D3DFVF_NORMAL) <> 0;

  // if using 32byte vertices, check fvf
//  if m_bForce32ByteFVF then
  if false then
  begin
    // force 32 byte vertices
    if (pMeshSysMem.GetFVF <> (D3DFVF_XYZ or D3DFVF_NORMAL or D3DFVF_TEX1)) then
    begin
      Result:= pMeshSysMem.CloneMeshFVF(pMeshSysMem.GetOptions, D3DFVF_XYZ or D3DFVF_NORMAL or D3DFVF_TEX1,
                                        m_pd3dDevice, pMeshTemp);
      if FAILED(Result) then Exit;

      pMeshSysMem := pMeshTemp;
    end;
  end
  // otherwise, just make sure that there is a normal mesh
  else if ((pMeshSysMem.GetFVF and D3DFVF_NORMAL) = 0) then
  begin
    Result:= pMeshSysMem.CloneMeshFVF(pMeshSysMem.GetOptions, pMeshSysMem.GetFVF or D3DFVF_NORMAL,
                                      m_pd3dDevice, pMeshTemp);
    if FAILED(Result) then begin Exit; end;

    pMeshSysMem := pMeshTemp;
  end;

  // Compute normals for the mesh, if not present
  if not bNormalsInFile then begin halt; D3DXComputeNormals(pMeshSysMem, nil); end;

//  m_pMesh:= pMeshSysMem;
//  pMeshSysMem:= nil; }
end;


procedure CModel.render;
var
  iCurFace: DWORD;
  iMaterial: Integer;

  dwFVF:                DWORD;
  cBytesPerVertex:      DWORD;
  iStrip:               DWORD;
   bla:PAnsiChar;
   matPos: TD3DXMatrix;
   i: integer;
   mapx, mapy: integer;
begin
    m_pd3dDevice.SetSamplerState(0, D3DSAMP_ADDRESSU, D3DTADDRESS_CLAMP);
    m_pd3dDevice.SetSamplerState(0, D3DSAMP_ADDRESSV, D3DTADDRESS_CLAMP);
    m_pd3dDevice.SetTextureStageState(1, D3DTSS_COLOROP, D3DTOP_DISABLE);
    m_pd3dDevice.SetTextureStageState(2, D3DTSS_COLOROP, D3DTOP_DISABLE);

  // Set and draw each of the materials in the mesh
  for iMaterial:= 0 to (m_dwNumMaterials - 1) do
  begin
    m_pd3dDevice.SetMaterial(m_pMeshMaterials[iMaterial]);
    m_pd3dDevice.SetTexture(0, m_pMeshTextures[iMaterial]);

    m_pMesh.DrawSubset(iMaterial);
  end;

end;



constructor CModelInstance.Create(const inst: CModel);
begin
    model := inst;

    position.x := 0;
    position.y := 0;
    position.z := 0;

    rotation.x := 0;
    rotation.y := 0;
    rotation.z := 0;

    isVisible := true;
    matrixChanged := true;
end;

destructor CModelInstance.destroy;
begin
end;

procedure CModelInstance.setPosition(x, y, z: single);
begin
    position.x := x;
    position.y := y;
    position.z := z;
    matrixChanged := true;
end;

procedure CModelInstance.move(x, y, z: single);
begin
    position.x := position.x + x;
    position.y := position.y + y;
    position.z := position.z + z;
    matrixChanged := true;
end;

procedure CModelInstance.setRotation(x, y, z: single);
begin
    rotation.x := x;
    rotation.y := y;
    rotation.z := z;
    matrixChanged := true;
end;

procedure CModelInstance.rotate(x, y, z: single);
begin
    rotation.x := rotation.x + x;
    rotation.y := rotation.y + y;
    rotation.z := rotation.z + z;
    matrixChanged := true;
end;

procedure CModelInstance.calculateMatrix;
var
   Keyboardstate: TKeyboardstate;
   matrix_x, matrix_y, matrix_z: TD3DMatrix;
   matrix: TD3DMatrix;
   trans: TD3DMatrix;
begin
    // Welt-Matrix
    D3DXMatrixIdentity(matrix);
//    m_pd3dDevice.SetTransform(D3DTS_WORLD, matrix);

    // Ansicht-Matrix
    D3DXMatrixRotationX(matrix_x, rotation.x);
    D3DXMatrixRotationY(matrix_y, rotation.y);
    D3DXMatrixRotationZ(matrix_z, rotation.z);

    D3DXMatrixMultiply(matrix, matrix, matrix_x);
    D3DXMatrixMultiply(matrix, matrix, matrix_y);
    D3DXMatrixMultiply(matrix, matrix, matrix_z);

    D3DXMatrixTranslation(trans, position.x, position.y, position.z);
    D3DXMatrixMultiply(matrix, matrix, trans);

    localMatrix := matrix;
    matrixChanged := false;
end;

procedure CModelInstance.render;
begin
     if matrixChanged then calculateMatrix;
     m_pd3dDevice.SetTransform(D3DTS_WORLD, localMatrix);
     model.render;
end;

end.