(*----------------------------------------------------------------------------*
 *                                                                            *
 *         M A R S                                                            *
 *                                                                            *
 *         by Jan Strter                                                     *
 *                                                                            *
 *----------------------------------------------------------------------------*)

 unit skybox;

interface

uses
  Windows, Direct3D9, D3DX9, dxerr9, global;

const
  D3DFVF_SKYBOX = D3DFVF_XYZ or D3DFVF_DIFFUSE or D3DFVF_TEX1;

type
  PSkyboxVertex = ^TSkyboxVertex;
  TSkyboxVertex = record
    position: TD3DXVector3;
    color: DWord;
    tu, tv: Single;
  end;

var
  skyvertices: Array [0..23] of TSkyboxVertex;

procedure createSkybox;
procedure renderSkybox(m_pd3dDevice: IDirect3DDevice9);


implementation


procedure createSkybox;

function makelvertex(x, y, z: single; color: dword; tu, tv: single): TSkyboxVertex;
var
   tmp: TSkyboxVertex;
begin
     tmp.position.x := x;
     tmp.position.y := y;
     tmp.position.z := z;
     tmp.color := color;
     tmp.tu := tu;
     tmp.tv := tv;
     result := tmp;
end;

const
    factor = 10;
begin

    //Vorn
    skyvertices[0] := makelvertex(-1*factor,-1*factor, 1*factor, d3dcolor_xrgb(255, 255, 255), 0, 1);
    skyvertices[1] := makelvertex(-1*factor, 1*factor, 1*factor, d3dcolor_xrgb(255, 255, 255), 0, 0);
    skyvertices[2] := makelvertex( 1*factor,-1*factor, 1*factor, d3dcolor_xrgb(255, 255, 255), 1, 1);
    skyvertices[3] := makelvertex( 1*factor, 1*factor, 1*factor, d3dcolor_xrgb(255, 255, 255), 1, 0);

    //Rechts
    skyvertices[4] := makelvertex( 1*factor, -1*factor,  1*factor, d3dcolor_xrgb(255, 255, 255), 0, 1);
    skyvertices[5] := makelvertex( 1*factor,  1*factor,  1*factor, d3dcolor_xrgb(255, 255, 255), 0, 0);
    skyvertices[6] := makelvertex( 1*factor, -1*factor, -1*factor, d3dcolor_xrgb(255, 255, 255), 1, 1);
    skyvertices[7] := makelvertex( 1*factor,  1*factor, -1*factor, d3dcolor_xrgb(255, 255, 255), 1, 0);

    //Hinten
    skyvertices[8] :=  makelvertex( 1*factor, -1*factor, -1*factor, d3dcolor_xrgb(255, 255, 255), 0, 1);
    skyvertices[9] :=  makelvertex( 1*factor,  1*factor, -1*factor, d3dcolor_xrgb(255, 255, 255), 0, 0);
    skyvertices[10] := makelvertex(-1*factor, -1*factor, -1*factor, d3dcolor_xrgb(255, 255, 255), 1, 1);
    skyvertices[11] := makelvertex(-1*factor,  1*factor, -1*factor, d3dcolor_xrgb(255, 255, 255), 1, 0);

    //Links
    skyvertices[12] := makelvertex(-1*factor, -1*factor, -1*factor, d3dcolor_xrgb(255, 255, 255), 0, 1);
    skyvertices[13] := makelvertex(-1*factor,  1*factor, -1*factor, d3dcolor_xrgb(255, 255, 255), 0, 0);
    skyvertices[14] := makelvertex(-1*factor, -1*factor,  1*factor, d3dcolor_xrgb(255, 255, 255), 1, 1);
    skyvertices[15] := makelvertex(-1*factor,  1*factor,  1*factor, d3dcolor_xrgb(255, 255, 255), 1, 0);

    //Oben
    skyvertices[16] := makelvertex(-1*factor, 1*factor,  1*factor, d3dcolor_xrgb(255, 255, 255), 0, 1);
    skyvertices[17] := makelvertex(-1*factor, 1*factor, -1*factor, d3dcolor_xrgb(255, 255, 255), 0, 0);
    skyvertices[18] := makelvertex( 1*factor, 1*factor,  1*factor, d3dcolor_xrgb(255, 255, 255), 1, 1);
    skyvertices[19] := makelvertex( 1*factor, 1*factor, -1*factor, d3dcolor_xrgb(255, 255, 255), 1, 0);

    //Unten
    skyvertices[20] := makelvertex(-1*factor, -1*factor, -1*factor, d3dcolor_xrgb(255, 255, 255), 0, 1);
    skyvertices[21] := makelvertex(-1*factor, -1*factor,  1*factor, d3dcolor_xrgb(255, 255, 255), 0, 0);
    skyvertices[22] := makelvertex( 1*factor, -1*factor, -1*factor, d3dcolor_xrgb(255, 255, 255), 1, 1);
    skyvertices[23] := makelvertex( 1*factor, -1*factor,  1*factor, d3dcolor_xrgb(255, 255, 255), 1, 0);
end;


procedure renderSkybox(m_pd3dDevice: IDirect3DDevice9);
var
   matPos: TD3DXMatrix;
   i: integer;
begin

    //Besondere Mappingregeln fr Texturen
    m_pd3dDevice.SetSamplerState(0, D3DSAMP_ADDRESSU, D3DTADDRESS_CLAMP);
    m_pd3dDevice.SetSamplerState(0, D3DSAMP_ADDRESSV, D3DTADDRESS_CLAMP);
    m_pd3dDevice.SetTextureStageState(1, D3DTSS_COLOROP, D3DTOP_DISABLE);
    m_pd3dDevice.SetTextureStageState(2, D3DTSS_COLOROP, D3DTOP_DISABLE);

    //ZBuffer ausschalten
    m_pd3dDevice.SetRenderState(D3DRS_ZENABLE, dWord(false));
    m_pd3dDevice.SetRenderState(D3DRS_ZWRITEENABLE, dWord(false));

    //Die Skybox mit der Kamera mitbewegen
    D3DXMatrixTranslation(matPos, -viewport.position.x, -viewport.position.y - 0.1, -viewport.position.z);
    m_pd3dDevice.SetTransform(D3DTS_WORLD, matPos);

    //Die Skybox rendern
    m_pd3dDevice.SetFVF(D3DFVF_SKYBOX);
//    m_pd3dDevice.SetStreamSource(0, skyVB, sizeof(TD3DLVertex));

    for i := 0 to 5 do
    begin
      m_pd3dDevice.SetTexture(0, skytextures[i]);
//      m_pd3dDevice.DrawPrimitive(D3DPT_TRIANGLESTRIP, i*4, 2);
      m_pd3dDevice.DrawPrimitiveUP(D3DPT_TRIANGLESTRIP, 2, skyvertices[i * 4], sizeOf(TSkyboxVertex));
    end;
end;

end.

