unit xfiles;

interface

uses
  Windows, Messages, Direct3D9, D3DX9, dxerr9,
  D3DUtil, DXUtil, sysutils, math, global;

type
  TXFile = class
    NumMaterials  : dWord;
    MeshMaterials : array of TD3DMaterial9;
    MeshTextures  : array of IDirect3DTexture9;
    Mesh : ID3DXMesh;
    procedure load(filename: string);
    procedure render;
    procedure cleanup;
  end;



implementation

procedure TXFile.load(filename: string);
var Adjacency, D3DXMtrlBuffer: PID3DXBuffer;
      d3dxMaterials: PD3DXMaterial;
      i: longint;
begin
  MeshMaterials := nil;
  MeshTextures  := nil;
  Mesh := nil;

  // Das Mesh laden
  Adjacency := nil;
  D3DXMtrlBuffer := nil;
  if failed(D3DXLoadMeshFromX(pChar(FileName), D3DXMESH_SYSTEMMEM, m_pd3dDevice, NIL, D3DXMtrlBuffer, NIL, @NumMaterials, Mesh)) then exit;
  Adjacency := nil;

  if NumMaterials = 0 then exit;

  d3dxMaterials := D3DXMtrlBuffer.GetBufferPointer;

  try
    SetLength(MeshMaterials, NumMaterials);
    SetLength(MeshTextures, NumMaterials);
  except
    exit;
  end;

  // Material und Texturen laden
  for i := 0 to NumMaterials - 1 do
    begin
      MeshMaterials[i] := d3dxMaterials^.MatD3D;

      MeshMaterials[i].Ambient := MeshMaterials[i].Diffuse;

      //Textur erzeugen
      MeshTextures[i] := nil;
      if (d3dxMaterials^.pTextureFilename = nil) or
         (failed(D3DXCreateTextureFromFile(m_pd3dDevice,
                 pchar(d3dxMaterials^.pTextureFilename),
                 MeshTextures[i])))
        then MeshTextures[i] := nil;

       inc(d3dxMaterials);
    end;

  D3DXMtrlBuffer := nil;
end;

// Diese Routine sieht zwar komplex aus, macht aber nicht viel besonders. Zunchst ldt sie das Mesh. Dann extrahiert sie die Materialeigenschaften und die Textureigenschaften und ldt die dazugehrigen Texturen. Ein Material in Direct3D gibt die Farbe des Vertex an.

// Naja viel spektaklrer wird es nicht. Die Renderprozedur geht nun durch jedes Material und zeichnet die dazugehrigen Meshdaten:

procedure TXFile.render;
var i: longint;
begin
  for i := 0 to NumMaterials - 1 do
  begin
    m_pd3dDevice.SetMaterial(MeshMaterials[i]);
    m_pd3dDevice.SetTexture(0, MeshTextures[i]);
    Mesh.DrawSubset(i);
  end;
end;

// Zum Schluss sollte man am Ende seines Programms die Texturen noch freigeben, um den Speicher wieder zu entlasten:

procedure TXFile.Cleanup;
var i: longint;
begin
  if (MeshMaterials <> nil) then MeshMaterials := nil;

  if (MeshMaterials <> nil) and (NumMaterials > 0) then
    for i := 0 to NumMaterials - 1 do
      if MeshTextures[i] <> nil
        then MeshTextures[i] := nil;
  MeshTextures := nil;

  if (Mesh <> nil) then Mesh := nil;
end;

end.
